//----------------------------------------------------------------------------
// 
//	file: matrix_window.h
// 
//	function: matrix_window class definition 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Jul  6 15:25:46 CDT 1998
// 
//	$Revision: 1.5 $ $Date: 2001/11/30 20:24:15 $
//	$Log: matrix_window.h,v $
//	Revision 1.5  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.4  2000/02/22 03:54:53  yschoe
//	*** empty log message ***
//
//	Revision 1.3  2000/02/08 11:25:21  yschoe
//	*** empty log message ***
//
//	Revision 1.2  1998/11/17 12:43:25  yschoe
//	added mouse event mask
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------
#ifndef __MATRIX_WINDOW_H
#define __MATRIX_WINDOW_H

#include <window.h>
#include <iostream.h>
#include "util.h"
#include "real.h"
#include "hsvrgb.h"
#include "simstat.h"

enum 	ColMode	{BW,COLOR,HSV,HSV_MULT};	// b/w or color cmap
enum	DimMode	{OneD,TwoD};	// matrix represented in 1D or 2D
enum	PlotMode {ITER,BATCH};  // one-by one feed (ITER) or plot in batch

//----------------------------------------------------------------------------
//	CLASS: matrix_window() 
//	  - displaying matrix
//----------------------------------------------------------------------------
class matrix_window : public pixmap_window{

  
protected:

  int		col, row;	// matrix dimension
  int 		xprev, yprev; // previous values for mouse motion events
  int		colprev, rowprev; // previous values for column / row


  REAL		*oned;		// points to the matrix (if in 1D repr.)
  REAL		**twod;		// points to the matrix (if in 2D repr.)
  int		*color_map;	// must be malloc()ed and set_color()ed.
  DimMode	dim_mode;	// OneD or TwoD
  ColMode	color_mode;	// BW or COLOR
  PlotMode	plot_mode;	// how input is feeded 
  REAL		scale;		// value x N scaling for color display
  REAL		**mult_mat;	// multiplication matrix

  void	alloc_colors();		// allocate colors in color_map
  void	init_gadgets();		// start up buttons, slides, etc.
				// only used by init()
  virtual void	init(int color_range, int color_mode, int col, int row);						// initvariables : only used by constructor.

public:
  int		color_range;	// number of colors to allocate
  simstat	*stats;

  	matrix_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int col, int row);
  	matrix_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int col, int row, REAL *oned);
  	matrix_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int col, int row, REAL **twod);
  	matrix_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int color_mode, int col, int row);
  	matrix_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int color_mode, int col, int row, REAL *oned);
  	matrix_window(window &parent, int w, int h, int x, int y, int bw,
		int color_range, int color_mode, int col, int row, REAL **twod);
  	~matrix_window();

  virtual void 	draw_interior();	// actual display routine
  virtual void	set_scale(REAL s);	// set scale and redraw
  virtual char  **disp_values();	// display values in a text-popup
  
  void	add_mult_matrix(REAL **mult);// add matrix to multiply cell-by-cell
  void  draw_circle(int cx, int cy, int radius, REAL color); // color=[0.0,1.0]
  void  draw_rectangle(int x1, int y1, int x2, int y2);
  void  draw_square(int cx, int cy, int radius);

  void  BPress_CB(XButtonEvent ev);
  void  Motion_CB(XMotionEvent ev);
  void  BRelease_CB(XButtonEvent ev);

  void register_stat(simstat *s) { stats = s; }

};
#endif
