//----------------------------------------------------------------------------
// 
//	file: parser.cc
// 
//	function: simple command line parser 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sat Aug  1 05:25:01 CDT 1998
// 
//	$Revision: 1.7 $ $Date: 2001/11/30 20:24:15 $
//	$Log: parser.cc,v $
//	Revision 1.7  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.6  1999/02/09 07:42:25  yschoe
//	replaced ansi-incompatible index with strchr
//
//	Revision 1.5  1998/10/26 03:53:13  yschoe
//	added next() function that returns the next token
//
//	Revision 1.4  1998/10/17 06:40:10  yschoe
//	fixed bug in scan_int() and scan_real(). fixed bug in rewind() (introduced
//	backup[])
//
//	Revision 1.3  1998/10/17 05:42:21  yschoe
//	implemented rest(char *).
//
//	Revision 1.2  1998/10/16 10:08:04  yschoe
//	introduced DELIM macro for delimiter.
//	introduced force_match(), rewind(), etc. rest() is not implemented.
//
//	Revision 1.1  1998/09/04 03:36:37  yschoe
//	fixed all warnings generated by -Wall
//
//
//----------------------------------------------------------------------------

#include "parser.h"
#include "real.h"

//----------------------------------------------------------------------------
//	parser() 
//	  - constructor
//----------------------------------------------------------------------------
parser::parser(const char *cmd) {

  command = new char[strlen(cmd)+1];
  backup  = new char[strlen(cmd)+1];

  strncpy(command,cmd,strlen(cmd)+1);
  strncpy(backup,cmd,strlen(cmd)+1);

  cmd_head = command;

  // start lexical analyser
  current = strtok(command, DELIM);

}
  

//----------------------------------------------------------------------------
//	~parser() 
//	  - destructor
//----------------------------------------------------------------------------
parser::~parser() {

  delete [] backup;
  delete [] command;

}


//----------------------------------------------------------------------------
//	match_param() 
//	  - match variable : scalar, vector, or matrix, with index ([], or [][])
//----------------------------------------------------------------------------
int parser::match_param(char *name, int *cols, int *rows) {

   char *tmp;
   int  b_count = 0;

   // lousy '[' check
   if (tmp=strchr(current,'[')) {
	b_count++; 
	tmp=tmp+1;
	if (tmp=strchr(tmp,'[')) {
		b_count++;
	}
   }

   *cols = -1; *rows = -1; // inticates it's a scalar
			// cols = n , rows = -1 : vector
			// cols = n , rows = m  : matrix

   char *left;
   switch (b_count) {
	case 0 : // scalar parameter
		strncpy(name,current,strlen(current)+1);
		current = strtok(NULL,DELIM);
		break;
	case 1 : // vector
		left = strstr(current,"[");
		strncpy(name,current,(int)(left-current));
		name[(int)(left-current)]='\0';
		sscanf(left,"[%d]",cols);
		current = strtok(NULL,DELIM);
		break;
	case 2 : // matrix
                left = strstr(current,"[");
                strncpy(name,current,(int)(left-current)); 
                name[(int)(left-current)]='\0';
		sscanf(left,"[%d]",cols);
                sscanf(left,"[%d][%d]",cols,rows);
		current = strtok(NULL,DELIM);
		break;
	default: cout << "parser::match_param - unknown type\n";
    }

    if (current == NULL) return 0; else return 1;

}

//----------------------------------------------------------------------------
//	match() 
//	  - match named token
//----------------------------------------------------------------------------
int parser::match(char *token) {

  if ((current != NULL) && (strcmp(current,token)==0)) {

	current = strtok(NULL,DELIM);
	return 1; 

  } else {

	//cout << "parser::match - token "
	//     << token << " not found\n" << endl;
	return 0;

  }

}

//----------------------------------------------------------------------------
//	force_match() 
//	  - match named token and print error message if no match
//----------------------------------------------------------------------------
int parser::force_match(char *token) {

  int ret_val = match(token);

  if (!ret_val) {
	cout << "parser::force_match - token "
	     << token << " not found\n" << endl;
  }
  return ret_val;
}
	
//----------------------------------------------------------------------------
//	scan_real() 
//	  - interpret it as a real value
//----------------------------------------------------------------------------
int parser::scan_real(REAL *r_val) {

  if (sscanf(current,"%f",r_val)) {
	current = strtok(NULL,DELIM);
	return 1;
  } else {
	return 0;
  }

}

//----------------------------------------------------------------------------
//	scan_int() 
//	  - interpret it as a int value
//----------------------------------------------------------------------------
int parser::scan_int(int *i_val) {

  if (sscanf(current,"%d",i_val)){
	current = strtok(NULL,DELIM);
	return 1;
  } else {
	return 0;
  }

}

//----------------------------------------------------------------------------
//	rewind()
//	  - rewind the strtok pointer to the beginning
//----------------------------------------------------------------------------
void parser::rewind() {

  strncpy(command,backup,strlen(backup)+1);
  current = strtok(command, DELIM);

}

//----------------------------------------------------------------------------
//	rest() 
//	  - the remaining string
//----------------------------------------------------------------------------
void parser::rest(char *dest) {

  int cur_len = strlen(current);
  // restore delimiter 
  current[cur_len] = ' ';
  strncpy(dest,current,strlen(current)+1);
  // restore '\0'
  current[cur_len] = '\0';
}

//----------------------------------------------------------------------------
//	cur() 
//	  - return current char pointer
//----------------------------------------------------------------------------
char *parser::cur() {

  return current;

}

//----------------------------------------------------------------------------
//	next() 
//	  - match next token
//----------------------------------------------------------------------------
int parser::next(char *dest) {

  if (current == NULL) 
    return 0;
  else {
    strncpy(dest,current,strlen(current)+1);
    current = strtok(NULL,DELIM);
    return 1; 
  }

}
