//----------------------------------------------------------------------------
// 
//	file: spikelayer2
// 
//	function: make spikelayer more robust to parameter change 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Nov 27 04:49:20 CST 1998
// 
//	$Revision: 1.1 $ $Date: 2001/11/30 20:24:15 $
//	$Log: spikelayer2-2.h,v $
//	Revision 1.1  2001/11/30 20:24:15  yschoe
//	Initial revision
//
//	Revision 1.2  2000/02/08 11:25:21  yschoe
//	*** empty log message ***
//
//	Revision 1.1  1999/02/09 07:42:25  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#ifndef __SPIKE_LAYER2_2_H
#define __SPIKE_LAYER2_2_H

#include "spikelayer.h"

//----------------------------------------------------------------------------
//	CLASS spikelayer
//	  - spiking lissom layer - derived from netlayer
//----------------------------------------------------------------------------
class spikelayer2_2 : public spikelayer{

  // General idea:
  //	Put decayed spike activity in this->output[x][y], so that
  //	it could be generically used with other types of netlayers.
  //	No spikes actually goes through the connection this way.
  //	Thus the postsynaptic potential is calculated for all 
  //	targets receiving from a certain neuron.

  // 00jun6 : keep the interface as is. change the activation code.
  //		push_spikes : now should 

private:

	REAL synapse_decay_mult;

	REAL 	lambda_exc_syn;
	REAL 	lambda_inh_syn;
	REAL 	lambda_int_syn;
	REAL 	lambda_loop;

	REAL	Heav(REAL x) { return ((x<=0.0) ? 0.0 : 1.0); }

	REAL	avg_rate;

public:
	REAL min_net_input, max_net_input;

	spikelayer2_2(char *name, int cols, int rows, int exc_rad, int inh_rad,
                 int max_inp_layers, netlayer *feeding,
                 int inp_cols, int inp_rows, int  aff_rad, 
		 REAL synapse_decay, REAL inhloop_decay, 
		 REAL theta=0.0, REAL tau=0.0,
		 int padded_flag=1);
       ~spikelayer2_2();

	void activate(int settling = 0);
	void print_stat(int i, int j);
	void print_stat(int cur_inp, int i, int j);

	void randomize_eta(void);
	void learn (void) { netlayer::learn(); }
	void reset_neurons(void);

};

#endif
