//----------------------------------------------------------------------------
// 
//	file: oja-lissom.cc
// 
//	function: use oja's rule in lissom.cc v1.1 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sun Aug 16 19:46:10 CDT 1998
// 
//	$Revision: 1.1 $ $Date: 1998/10/16 10:35:21 $
//	$Log: oja-lissom.cc,v $
//	Revision 1.1  1998/10/16 10:35:21  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#include <iostream.h>
#include <window.h>
#include <nnview/netlayer.h>
#include <nnview/plot_window.h>
#include <nnview/matrix_window.h>
#include <nnview/tool_window.h>
#include <nnview/net_window.h>
#include <nnview/simulog.h>
#include <nnview/real.h>
#include <nnview/coord.h>

class oja_netlayer : public netlayer {

public:

        oja_netlayer(char *name, int cols, int rows, int exc_rad, int inh_rad,
                 int max_inp_layers, netlayer *feeding=NULL,
                 int inp_cols=0, int inp_rows=0, int  aff_rad=0);
        ~oja_netlayer();

	inline virtual REAL delta(int col, int row, int cur_inp, int inp_col,
                        int inp_row);

};

oja_netlayer::oja_netlayer(char *name, int cols, int rows, 
		 int exc_rad, int inh_rad,
                 int max_inp_layers, netlayer *feeding=NULL,
                 int inp_cols=0, int inp_rows=0, int  aff_rad=0) 
: netlayer (name,cols,rows,exc_rad,inh_rad,max_inp_layers,feeding,inp_cols,inp_rows,aff_rad) {
	// do nothing
}

oja_netlayer::~oja_netlayer() {
	// do nothing
}

inline REAL oja_netlayer::delta(int col, int row, int cur_inp, int inp_col,
                        int inp_row) {
  int offset_x, offset_y;

  offset_x = rf_ranges[cur_inp][col][row].lowx;
  offset_y = rf_ranges[cur_inp][col][row].lowy;

  return ( alpha_affs[cur_inp] 
	  * output[col][row] 
	  * (feedings[cur_inp]->output[inp_col][inp_row]
	     - (output[col][row]
		* wgt[col][row].aff_ws[cur_inp][inp_col-offset_x][inp_row-offset_y]
	       )
	    )
	 );
}

main_window     *mw;
plot_window     *pw ;
plot_window     *pw2 ;
plot_window     *pw3 ;
matrix_window   *matw;
matrix_window   *matw2;
tool_window     *tw;
net_window	*nw;
callback_button *runit;

int             steps;
int 		iters = 101;
simulog		*netlog;

oja_netlayer	*input;
oja_netlayer	*map;

coord		**cog;
coord		**ideal;


int N = 12;
int M = 3;
REAL randomness=0.5;

//----------------------------------------------------------------------------
//      setup() 
//        - bplayer : setup network for simulation
//----------------------------------------------------------------------------
void setup(char *configfile) {

  netlog = new simulog("lissom.cc","lissom.log");

  // input layer
  cout << "Initializing input layer.\n";
  input = new oja_netlayer("input",1,M,0,0,0);
  
  // map layer
  cout << "Initializing map layer.\n";
  map   = new oja_netlayer("map",N,N,4,(int)(N*1.0),1,input,1,M,M);

  // set parameter values
  //   read from file.
  FILE *conffp = fopen(configfile,"r");
  char buff[100];

  // read parameters
  int count = 0;
  while (!feof(conffp)) {
    	fgets(buff,100,conffp);
	if (feof(conffp)) { break; }
        map->set_parameter(buff);
	count ++;
  }
  cout << "lissom:setup() " << count << " commands processed\n";
  fclose(conffp);

  // init weights
  map->init_weights(0.0,1.0);

  // normalize weight to cartesian coord
  REAL cosproduct;
  REAL w[2];
  for (int i=0; i<map->cols; ++i) {
    for (int j=0; j<map->rows; ++j) {
      cosproduct=1.0;
      w[0] = ((1.0-randomness)*i/(REAL)(map->cols-1) 
	   + randomness*drand48()); 
      w[1] = ((1.0-randomness)*j/(REAL)(map->rows-1)
	   + randomness*drand48()); 
      map->wgt[i][j].aff_ws[0][0][0] = cos(w[0])*cos(w[1]);
      map->wgt[i][j].aff_ws[0][0][1] = sin(w[0])*cos(w[1]);
      map->wgt[i][j].aff_ws[0][0][2] = sin(w[1]);
    }
  }
  
  // map->normalize_weights();

  // set up ideal grid
  // ideal = NULL;
  ideal = new coord* [N];
  for (int i=0; i<N; ++i) 
	ideal[i] = new coord [N];

  for (int i=0; i<map->cols; ++i) {
    for (int j=0; j<map->rows; ++j) {
       w[0] = (REAL)i/((REAL)map->cols-1);
       w[1] = (REAL)j/((REAL)map->cols-1);
       ideal[i][j].x = w[0];
       ideal[i][j].y = w[1];
    }
  }

  // set up COG (center of gravity) map
  cog = new coord* [N];
  for (int i=0; i<N; ++i) 
	cog[i] = new coord [N];

}


//----------------------------------------------------------------------------
//	calc_cog() 
//	  - calculate center of gravity coordinates
//----------------------------------------------------------------------------
void calc_cog(coord **cog, oja_netlayer *map, int cur_inp_layer){

  for (int x=0; x<map->cols; ++x) {

     for (int y=0; y<map->rows; ++y) {

	// calculate sums for center of gravity 
        REAL sum = 0.0;
        REAL x_sum = 0.0;
        REAL y_sum = 0.0;
	/*
	for (int i=map->rf_ranges[cur_inp_layer][x][y].lowx;
		 i<=map->rf_ranges[cur_inp_layer][x][y].highx; ++i) {
	   for (int j=map->rf_ranges[cur_inp_layer][x][y].lowy;
		    j<=map->rf_ranges[cur_inp_layer][x][y].highy; ++j) {
		int rf_x = i-map->rf_ranges[cur_inp_layer][x][y].lowx;
		int rf_y = j-map->rf_ranges[cur_inp_layer][x][y].lowy;
		REAL wgt = map->wgt[x][y].aff_ws[cur_inp_layer][rf_x][rf_y];
		sum += wgt;
		x_sum += i*wgt;
		y_sum += j*wgt;
	   }
      	}	
	*/
		
	// get the 3D weights
        REAL x_wgt = map->wgt[x][y].aff_ws[cur_inp_layer][0][0]; //x_sum/sum;
        REAL y_wgt = map->wgt[x][y].aff_ws[cur_inp_layer][0][1]; //y_sum/sum;
        REAL z_wgt = map->wgt[x][y].aff_ws[cur_inp_layer][0][2]; //y_sum/sum;

	// now save the actual c.o.g. after transforming back to 2D
 	cog[x][y].x = asin(y_wgt/cos(asin(z_wgt)));
	cog[x][y].y = asin(z_wgt); ///cos(asin(x_wgt)));

     }

  }

}

//----------------------------------------------------------------------------
//      run() 
//        - bplayer : run a certain number of backprop learning
//----------------------------------------------------------------------------
void run () {

  char str[100];

  // for the number of given steps
  for (int i=0; i<steps; ++i) {

    // set input and normalize
    REAL sum=0.0;
    REAL x_inp = drand48();
    REAL y_inp = drand48();
    input->output[0][0] = cos(x_inp)*cos(y_inp);
    input->output[0][1] = sin(x_inp)*cos(y_inp);
    input->output[0][2] = sin(y_inp);

    // clear map activity
    map->reset_output();

    // activate and settle
    for (int k=0; k<15; ++k) {
	map->activate();
	// display
        if (i%10==0) {

		// calculate c.o.g.
		calc_cog(cog,map,0);
		matw->redraw();
    		nw->redraw();
	}
	set_color(white);
	XFillRectangle(display,mw->Win,gc_copy,
		0,mw->height-20,mw->width,20);	
	set_color(black);
	sprintf(str,"settle=%d",k);
	mw->DrawString(0,mw->height-10,str);
	sprintf(str,"iter=%d",i);
	mw->DrawString(0,mw->height,str);
	mw->redraw();
	XFlush(display);
    }

    // learn
    map->learn();
    // oja_netlayer: this is not necessary
    // map->normalize_weights();

    // update sigmoid
    if (i<1500) {
      	map->sig_low += (map->sig_high-map->sig_low)/2.0/(steps/20.0);
     	// cout << "new sig_low = " << map->sig_low << endl; 
      	//map->sig_high -= (map->sig_high-map->sig_low)/4.0/(steps/10.0);
    }

    // reduce learing rate
    /*
    if (i%600==0 && i!=0) { 
	if (map->alpha_affs[0]>0.001) {
      	REAL new_alpha = map->alpha_affs[0];
      	map->alpha_affs[0] = new_alpha * 0.1;
      	map->alpha_lat_exc = map->alpha_lat_exc * 0.1 ;
      	map->alpha_lat_inh = map->alpha_lat_inh * 0.1 ;
       	cout << "new alpha = " << new_alpha << endl;     
	}
    }
    */
	
    // reduce radius
    if (i==300) {
	cout << "lissom.cc:run():  reducing radius.\n";
	map->resize_lat_exc_rad(2);
    }
    if (i==600) {
	cout << "lissom.cc:run():  reducing radius.\n";
	map->resize_lat_exc_rad(1);
    }

    // kill connections
    if (i%1000==0 && i!=0) {
	cout << "lissom.cc:run():  killing lat inh weights.\n";
	map->kill_lat_weights(0.001);
    }

  }

}

//----------------------------------------------------------------------------
//	main() 
//	  - the MAIN
//----------------------------------------------------------------------------
int main(int argc, char *argv[]) {

  steps = 20000;

  // check args
  if (argc != 2) {
	cout << "usage: " << argv[0] << " <configfilename>\n";
 	exit(0);
  }

  // setup network
  setup(argv[1]);

  // setup the windows
  mw = new main_window(argv[0],480,260);
  
  matw = new matrix_window(*mw,240,240,0,0,0,60,COLOR,N,N,map->output);  

  runit = new callback_button(*mw,"Run",run,
                mw->width/4,20,mw->width/2,mw->height-20);
 
  nw = new net_window(*mw,240,240,240,0,0,5,COLOR,N,N,0.0,1.0,0.0,1.0,cog,ideal);

  new quit_button(*mw,mw->width/4,20,mw->width*3/4,mw->height-20);

  // start main loop
  mw->main_loop();

  delete netlog;

}
