//----------------------------------------------------------------------------
// 
//	file: slissom2.cc
// 
//	function: modify slissom algorithm to be more robust to parameters 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Nov 30 22:45:30 CST 1998
// 
//	$Revision: 1.4 $ $Date: 2000/01/30 14:55:55 $
//	$Log: slissom2.cc,v $
//	Revision 1.4  2000/01/30 14:55:55  yschoe
//	*** empty log message ***
//
//	Revision 1.3  1999/12/18 05:44:03  yschoe
//	*** empty log message ***
//
//	Revision 1.2  1999/02/09 07:56:44  yschoe
//	removed cog calc and use the one provided by netlayer
//	and added wgt dump interfaces
//
//	Revision 1.1  1999/01/01 07:18:41  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// 
//	file: slissom.cc
// 
//	function: test spikelayer2 class 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Sun Sep 27 05:49:27 CDT 1998
// 
//	Revision: 1.6  Date: 1998/11/09 13:23:42 
//	Log: slissom.cc,v 
//	Revision 1.6  1998/11/09 13:23:42  yschoe
//	*** empty log message ***
//
//	Revision 1.5  1998/10/26 04:02:22  yschoe
//	1. weight display windows added
//	2. network weight setup parameters added to blackboard for easier
//	   customization with the conf file
//	3. filename handling got better by the use of filename class.
//	4. simulation environment and network image separated.
//	5. command hooks used with the simulation environment.
//
//	Revision 1.4  1998/10/18 06:14:58  yschoe
//	*** empty log message ***
//
//	Revision 1.3  1998/10/16 10:35:21  yschoe
//	added randomness, and added a couple of display windows.
//
//	Revision 1.2  1998/10/05 19:59:23  yschoe
//	now capable of  gaussian initial weights.
//
//	Revision 1.1  1998/10/02 09:24:16  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// 
//	file: lissom.cc
// 
//	function: defines and implements LISSOM 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Wed Aug  5 18:04:55 CDT 1998
// 
//	Revision: 1.15 Date: 1998/09/14 06:03:04 
//	Log: lissom.cc,v -- see rlog
//
//----------------------------------------------------------------------------
#define lissom_version "slissom2.cc $Revision: 1.4 $"

#include <iostream.h>
#include <signal.h>

#include <nnview/spikelayer2.h>
#include <nnview/simulog.h>
#include <nnview/real.h>
#include <nnview/coord.h>
#include <nnview/geninput.h>
#include <nnview/simenv.h>
#include <nnview/filename.h>
#include <nnview/simstat.h>
#include <nnview/inputlist.h>

#ifndef LissomCUI
#include <window.h>
#include <nnview/plot_window.h>
#include <nnview/matrix_window.h>
#include <nnview/tool_window.h>
#include <nnview/net_window.h>
#include <nnview/disputil_window.h>
#include <nnview/gdb_visualize.h>
#include <nnview/affw_window.h>
#include <nnview/latw_window.h>
#endif


//----------------------------------------------------------------------------
// Network topology and simulation parameters
simenv	*environment;
simstat *stats;
int 	N = 11;
int 	M = 11;
int 	INH_RAD = 11;
int 	EXC_RAD = 7;
int 	RF_RAD = 9;
int 	GRAN = 5;
int 	WGT_GRAN = 2;
REAL 	LAMBDA_SYN = 0.3;
REAL 	LAMBDA_INH = 0.1;
REAL 	randomness=1.0;
REAL 	aff_setup_decay=0.5;
REAL 	lat_inh_setup_decay=1.0;
REAL 	lat_exc_setup_decay=3.0;
REAL 	sigmax=1.0;
REAL 	sigmay=1.0;
REAL 	wgt_disp_scale=1.0;
int 	disp_rate = 1;
int 	wgt_disp_rate = 1000;
int 	iters = 0;
int 	iters_per_run = 10;
int 	tsettle = 9;
int 	test_flag = 0;
char 	conffilename[100]; // config file base name
char 	buffname[100]; // config file base name
int     save_rate = 5000;
int     wgt_disp_flag = 0;
int	instar_norm = 0;
REAL  scaleup=1.0;
int   hsv_spike_flag=0;
int   uniform_wgt_flag=0;
int     cur_x, cur_y; // training input location
int 	cur_t;


REAL  synapse_decay = 0.4;
REAL  inhloop_decay = 0.5;
REAL  synapse_decay2 = 0.4;
REAL  inhloop_decay2 = 0.5;


int     steps;
int 	t=0;
simulog	*netlog;

netlayer	*input;
spikelayer2	*map;

coord		**ideal;

geninput	*input_generator; // input generator class
inputlist      *input_spec_list;         // list of input specs

// End
//----------------------------------------------------------------------------

// prototypes
void run();
void affdisp();
int  call_add_spec_rect(int argc, char *argv[]);
int  call_add_spec_ori_gaus(int argc, char *argv[]);

//----------------------------------------------------------------------------
// GUI stuff
#ifndef LissomCUI

class mymain_window : public main_window {

public:
  mymain_window(char *name, int x, int y) : main_window(name,x,y) {
     selection_mask |= ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
     polling_mode = False;
  }
  ~mymain_window(){};
  void polling_handler() {
	iters_per_run=1;
	run();
  }	
	
};

mymain_window     *mw;

plot_window     *pw ;
plot_window     *pw2 ;
plot_window     *pw3 ;

matrix_window   *matw;
matrix_window   *inpw;
matrix_window   *matw2;
matrix_window   *spikew;
matrix_window   *avgw;

tool_window     *tw;

affw_window     *affdispw;
latw_window     *latinhdispw;
latw_window     *latexcdispw;
latw_window     *latcombdispw;

net_window	*nw;

callback_button *runit;
callback_button *testit;
callback_button *saveit;
callback_button *affdispit;
callback_button *commandit;

// Matrices for displaying
REAL		**spikeact;

#endif
// End: GUI stuff
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//	chop() 
//	  - chop last entry in a string
//----------------------------------------------------------------------------
void chop(char *str) {

  str[strlen(str)-1] = '\0';

}

//----------------------------------------------------------------------------
//      repeat_print() 
//        - print given string repetedly for given number of times
//----------------------------------------------------------------------------
void repeat_print(const char *str, int num) {

   for (int i=0; i<num; ++i)
        cout << str;

   cout << endl;
}

#ifndef LissomCUI
//----------------------------------------------------------------------------
//	disp_spikes() 
//	  - display spikes
//----------------------------------------------------------------------------
void disp_spikes() {

  static int count;
  static int pwin_flag=0;

  if (test_flag) {
    cout << "spikes: " ;
    for (int i=0; i<map->cols; ++i)
       for (int j=0; j<map->rows; ++j)  {
          spikeact[i][j] = (REAL)(map->spikeact_q[i][j]&0x1);
          cout << ((int)spikeact[i][j]) << " ";
       }
    cout << endl;
  }

  for (int i=0; i<map->cols; ++i) 
     for (int j=0; j<map->rows; ++j) 
	spikeact[i][j] = (REAL)(map->spikeact_q[i][j]&0x1);

  if (test_flag) {
    cout << "mua " << count << " " ;
    for (int i=0; i<stats->get_num_specs(); ++i) {
        cout << stats->sum_area(i) << " ";
        matw->draw_circle(stats->get_cx(i),stats->get_cy(i),
                stats->get_cradius(i),1.0);
        spikew->draw_circle(stats->get_cx(i),stats->get_cy(i),
                stats->get_cradius(i),1.0);
    }
    cout << endl;

    for (int i=0; i<stats->get_num_specs(); ++i) {
      cout << "neuron_stat" << i << " " << count << " ";
      map->print_stat(0,stats->get_cx(i),stats->get_cy(i));
    }
    count++;

  } else {

     // if (count%disp_rate==0) {
     //  cout << "neuron_stat" << 0 << " " << count << " ";
     //  map->print_stat(0,cur_x,cur_y);
     // }

     count++;


  }

  spikew->redraw();
  XFlush(display);  

}
#endif

//----------------------------------------------------------------------------
//      call_add_spec_rect() 
//        - add rectangular input to spec list
//----------------------------------------------------------------------------
int call_add_spec_rect(int argc, char *argv[]) {

        input_spec_list->call_add_spec_rect(argc, argv);
        return 1;

}

//----------------------------------------------------------------------------
//      call_add_spec_ori_gaus() 
//        - add oriented gaussian input to spec list
//----------------------------------------------------------------------------
int call_add_spec_ori_gaus(int argc, char *argv[]) {

        input_spec_list->call_add_spec_ori_gaus(argc, argv);
        return 1;

}

//----------------------------------------------------------------------------
//      call_add_mua_spec_circle() 
//        - add mua spec -- circular area
//----------------------------------------------------------------------------
int call_add_mua_spec_circle(int argc, char *argv[]) {

        stats->call_add_spec_circle(argc, argv);
        return 1;

}
//----------------------------------------------------------------------------
//      call_add_mua_spec_sqr() 
//        - add mua spec -- square area
//----------------------------------------------------------------------------
int call_add_mua_spec_sqr(int argc, char *argv[]) {

        stats->call_add_spec_sqr(argc, argv);
        return 1;

}

//----------------------------------------------------------------------------
//      call_add_mua_spec_bbox() 
//        - add mua spec -- bounded box area
//----------------------------------------------------------------------------
int call_add_mua_spec_bbox(int argc, char *argv[]) {

        stats->call_add_spec_sqr(argc, argv);
        return 1;

}

//----------------------------------------------------------------------------
//      setup() 
//        - setup network for simulation
//----------------------------------------------------------------------------
void setup(int argc, char *argv[], char *configfile) {


  // copy config filename to conffilename
  netlog = new simulog(argv[0],filename::swap_ext(buffname,conffilename,"log"));

  netlog->append("command line %s %s ...",argv[0],argv[1]);
  netlog->append("code version: %s",lissom_version);
  netlog->append("code version: %s",netlayer_version);

  //
  // 1. Setup simulation environment
  //
  environment = new simenv("slissom environment",100);

  // 1.1 add local simulation parameters to the 
  // 	 blackboard 
  environment->add("N",&N);
  environment->add("M",&M);
  environment->add("INH_RAD", &INH_RAD);
  environment->add("EXC_RAD", &EXC_RAD);
  environment->add("RF_RAD", &RF_RAD);
  environment->add("LAMBDA_SYN", &LAMBDA_SYN);
  environment->add("LAMBDA_INH", &LAMBDA_INH);
  environment->add("GRAN",&GRAN);
  environment->add("WGT_GRAN",&WGT_GRAN);
  environment->add("randomness",&randomness);
  environment->add("aff_setup_decay",&aff_setup_decay);
  environment->add("lat_exc_setup_decay",&lat_exc_setup_decay);
  environment->add("lat_inh_setup_decay",&lat_inh_setup_decay);
  environment->add("wgt_disp_scale",&wgt_disp_scale);
  environment->add("sigmax",&sigmax);
  environment->add("sigmay",&sigmay);
  environment->add("iters",&iters);
  environment->add("disp_rate",&disp_rate);
  environment->add("wgt_disp_rate",&wgt_disp_rate);
  environment->add("iters_per_run",&iters_per_run);
  environment->add("tsettle",&tsettle);
  environment->add("save_rate",&save_rate);
  environment->add("wgt_disp_flag",&wgt_disp_flag);
  environment->add("instar_norm",&instar_norm);


  // 1.2 read from file and set up hook tables
  // 	 input layer
  int count = environment->parse_file(configfile);

  // 1.3 log the dimension and stuff
  netlog->append("N = %d (map NxN)",N);
  netlog->append("M = %d (input MxM)",M);
  netlog->append("randomness = %f",randomness);

  //
  // 2. Set up map layers, and register maps in the environment, then
  //	run hooks
  //
  
  // 2.1 Input layer
  cout << "Initializing input layer.\n";
  input = new netlayer("input",M,M,0,0,0);
  
  // 2.2 SLISSOM layer
  cout << "Initializing map layer.\n";
  map   = new spikelayer2("map",N,N,EXC_RAD,INH_RAD,1,input,M,M,RF_RAD,
		LAMBDA_SYN,LAMBDA_INH,0.4,0.5);
 
  // 2.2.1 check if correct type of normalization is used.
  if (map->is_norm_by_sum()) {
      cout    	<< "***************** Warning ***************************\n"
      		<< "slissom : wrong normalization used in netlayer class.\n"
              	<< "    recompile the libnnview library and slissom.\n" 
          	<< "*****************************************************\n"
		<< endl;
  } 
  cout << "Normalization : " << 2.0 << " " << map->calc_norm_sum(2.0) << endl;

  // 2.3 add the spike layer to the environment, and add commands
  environment->add_obj("lissom",map,"slissom layer");

  environment->add_alias("slissom","lissom");

  environment->add_cmd("lissom->call_resize_lat_exc_rad","slissom",
	&spikelayer2::call_resize_lat_exc_rad, 1, 
	" <I:radius>");

  environment->add_cmd("slissom->call_resize_lat_exc_rad","slissom",
	&spikelayer2::call_resize_lat_exc_rad, 1, 
	" <I:radius>");

  environment->add_cmd("lissom->call_update_sig","slissom",
        &spikelayer2::call_update_sig, 2,
        " <R:sig_low> <R:sig_high>");

  environment->add_cmd("slissom->call_update_sig","slissom",
        &spikelayer2::call_update_sig, 2,
        " <R:sig_low> <R:sig_high>");

  environment->add_cmd("lissom->call_dump_wgts_range","slissom",
        &spikelayer2::call_dump_wgts_range, 4,
        " <I:start_col> <I:start_row> <I:end_col> <I:end_row>");

  environment->add_cmd("lissom->call_dump_wgt_sums_range","slissom",
        &spikelayer2::call_dump_wgt_sums_range, 4,
        " <I:start_col> <I:start_row> <I:end_col> <I:end_row>");

  environment->add_cmd("slissom->call_save_or_pref","slissom",
        &spikelayer2::call_save_or_pref, 2,
        " <I:cur_input> <Str:filename>");

  // input_spec_list 
  environment->add_cmd( "call_add_spec_rect",&call_add_spec_rect, 3,
        " <I:cx> <I:cy> <I:radius>");

  environment->add_cmd( "call_add_spec_ori_gaus",&call_add_spec_ori_gaus, 5,
        " <I:cx> <I:cy> <R:sigmax> <R:sigmay> <R:angle>");

  // simenv mua spec 
  environment->add_cmd( "call_add_mua_spec_circle",&call_add_mua_spec_circle, 3,
        " <I:cx> <I:cy> <I:cradius>");

  environment->add_cmd( "call_add_mua_spec_bbox",&call_add_mua_spec_bbox, 4,
        " <I:x1> <I:y1> <I:x2> <I:y2>");

  environment->add_cmd( "call_add_mua_spec_sqr",
        &call_add_mua_spec_sqr, 3,
        " <I:cx> <I:cy> <I:radius>");

  // 2.4 execute init_hooks before proceeding
  environment->run_hooks_init();

  //
  // 3. init weights
  //
  map->init_weights(0.0,randomness);
  map->setup_weight(NETWGT_CIRCULAR_AFF,RF_RAD,randomness);
  map->setup_weight(NETWGT_INH,(REAL)lat_inh_setup_decay,randomness);
  map->setup_weight(NETWGT_EXC,(REAL)lat_exc_setup_decay,randomness);
  // map->kill_overlapping_inh_w();
  if (instar_norm) {
	cerr << "Instar normalization" << endl;
  	map->normalize_instar_weights();
  } else {
  	map->normalize_weights();
  }
  

  // 
  // 4. set up ideal grid and COG matrix : todo -- this may go into netlayer
  //	class
  //
  
  // 4.1 Ideal grid
  ideal = new coord* [N];
  for (int i=0; i<N; ++i) 
	ideal[i] = new coord [N];

  REAL mag;

  mag = (M-2*map->aff_rads[0])/(REAL)N;
  for (int i=0; i<map->cols; ++i) {
    for (int j=0; j<map->rows; ++j) {
       ideal[i][j].x = (REAL) i*mag + map->aff_rads[0];
       ideal[i][j].y = (REAL) j*mag + map->aff_rads[0];
    }
  }

}

//----------------------------------------------------------------------------
//      run() 
//        - bplayer : run a certain number of backprop learning
//----------------------------------------------------------------------------
void run () {

  char str[100];
  int i;

  cout << "environment->list_scalar() <<<<<<<<<<<<<<<<<<<<<<<<<<<<" << endl;
  environment->list_scalar();
  cout << "map->net_bb->list_scalar() <<<<<<<<<<<<<<<<<<<<<<<<<<<<" << endl;
  map->net_bb->list_scalar();

  // for the number of given steps
  for (i=t; i<t+iters_per_run; ++i) {

    cur_t = i;

    // Run hooks
    environment->run_hooks_before(i);

    //
    // 1. clear input
    //
    REAL sum=0.0;
    input_generator->clear_input();

    /* Gaussian */
    if (test_flag) {

        // use the registered input specs for input generation
        input_spec_list->add_inputs();

    } else {

        input_generator->clear_input();
	map->clear_activity();

        cur_x = (int)(drand48()*M);
        cur_y = (int)(drand48()*M);

        if (sigmax>1.0) {
          input_generator->add_input(cur_x,cur_y,
                        (REAL)sigmax, (REAL)sigmay, (REAL)(drand48()*360.0));
        } else {
          input_generator->add_input(cur_x,cur_y,1);
        }

        if (scaleup>1.0)
                matrix<REAL>::scaleup(input->output,M,M,scaleup);

    }

    if (0) {
          input_generator->clear_input();
    }


 
#   ifndef LissomCUI
    if (i%disp_rate==0) { inpw->redraw(); XFlush(display); }
#   endif

    // clear map activity
    // map->reset_output();

    // activate and settle
    int settling = test_flag ? 1 : tsettle;
    for (int k=0; k<settling ; ++k) {

	if (test_flag) {
	  if (i==t) 
		map->activate(0);
	  else 
		map->activate(1);
 	} else {
	  map->activate(k); 
	}

	// display
        if (i%disp_rate==0) {
#	    	ifndef LissomCUI
		// calculate c.o.g.  
		map->calc_cog(0); nw->redraw(); mw->redraw(); disp_spikes();
		matw->redraw();
#	    	else
		if (k==0) { cout << "iter " << i << endl; }
#	    	endif
	}	
#       ifndef LissomCUI
	set_color(white); 
	XFillRectangle(display,mw->Win,gc_copy,
		0,mw->height-20,mw->width,20);	
	set_color(black); sprintf(str,"settle=%d",k);
	mw->DrawString(0,mw->height-10,str); sprintf(str,"iter=%d",i);
	mw->DrawString(0,mw->height,str); XFlush(display);
    	if (i%disp_rate==0) {
    		avgw->redraw(); XFlush(display);
    	}
#   endif

    } // end of settling loop

    // learn weights
    map->learn();
    if (instar_norm) {
	cerr << "Instar normalization" << endl;
  	map->normalize_instar_weights();
    } else {
  	map->normalize_weights();
    }

    // Run hooks
    environment->run_hooks_after(i);
    
    // log every 1000 iterations
    if (i%disp_rate==0) {
	netlog->append("iter %d reached.",i);
    }
    if (i%wgt_disp_rate==0 && i!=0) {
	affdisp();
    }
    if (i%save_rate==0) {
      char imgbasename[200];
      char curimgfile[200];
      filename::swap_ext(imgbasename,conffilename,"");
      sprintf(curimgfile,"%s%d.sav",imgbasename,i);
      map->net_bb->save(curimgfile);
    }


  } // end of big loop

  // set start of next run
  t = i;

}


//----------------------------------------------------------------------------
//	test() 
//	  - testing mode
//----------------------------------------------------------------------------
void test() {

  test_flag = 1;

  run();

}

//----------------------------------------------------------------------------
//	save() 
//	  - save blackboard
//----------------------------------------------------------------------------
void save() {

  cout << "lissom.cc: saveit() : saving to lissom.sav\n";
  map->net_bb->save(filename::swap_ext(buffname,conffilename,"sav"));
  //dispmat(map->output,36,36,5);

}

//----------------------------------------------------------------------------
//	affdisp() 
//	  - popup and display afferent weights
//----------------------------------------------------------------------------
void affdisp() {

  char or_fname[256];

#ifndef LissomCUI
  static int toggled=0;

  switch(wgt_disp_flag) {
  case 0: break;
  case 5:
  case 1: affdispw->set_scale(wgt_disp_scale); break;
  case 6:
  case 2: latinhdispw->set_scale(wgt_disp_scale); break;
  case 3: latexcdispw->set_scale(wgt_disp_scale); break;
  case 4: latcombdispw->redraw(); break;
  }

  if (!toggled) {
      switch(wgt_disp_flag) {
      case 0: break;
      case 5:
      case 1: affdispw->toggle(); break;
      case 6:
      case 2: latinhdispw->toggle();break;
      case 3: latexcdispw->toggle();break;
      case 4: latcombdispw->toggle();break;
      }
      toggled=1;
  } else {
      switch(wgt_disp_flag) {
      case 0: break;
      case 5:
      case 1: affdispw->redraw(); break;
      case 6:
      case 2: latinhdispw->redraw(); break;
      case 3: latexcdispw->redraw(); break;
      case 4: latcombdispw->redraw(); break;
      }
  }	

#endif

  if (wgt_disp_flag < 5) {

  map->calc_or_pref(0,MAX(sigmax/2.0,1.0),MAX(sigmay/2.0,1.0));
  map->print_or_pref();
  sprintf(or_fname,
	"%s%06d.or",
	filename::swap_ext(buffname,conffilename,""),
	cur_t);
  map->save_or_pref(0,or_fname);

  repeat_print("0 ",N);

  }

}

//----------------------------------------------------------------------------
//	command() 
//	  - read input and run command
//----------------------------------------------------------------------------
void command() {

  char command[256];

  cerr << "Entering command shell" << endl;

  cout << "slissom command shell : type 'return' to end command session."<< endl;
  cout << "slissom> ";
  fgets(command,255,stdin);
  while (strncmp(command,"return",6)!=0) {
	// check if it is a local command
	if (strncmp(command,"exec",3)==0) {
		run();
	} else if (strncmp(command,"disp_wgt",8)==0) {
		affdisp();
	} else if (strncmp(command,"save",4)==0) {
		save();
	} else if (strncmp(command,"exit",4)==0) {	
		exit(0);
	} else {
		environment->parse(command);
	} 
  	cout << "slissom> ";
  	fgets(command,255,stdin);
  }

}

#ifndef LissomCUI
//----------------------------------------------------------------------------
//	gdb_dispmat() 
//	  - for debugging, display arbirtaty map
//----------------------------------------------------------------------------
void gdb_dispmat(REAL **mat, int cols, int rows, int gran) {

  disputil_window *dw = new disputil_window("gdb vis",mat,rows,cols,gran);
  unmap_button *dwclose = new unmap_button(*dw,"close",cols*gran,20,0,rows*gran);  
  dw->toggle();

}
#endif


//----------------------------------------------------------------------------
//      handler() 
//        - handle SIGINT and give command prompt
//----------------------------------------------------------------------------
void handler(int stat) {

  cout << "SIGINT received. Entering command mode." << endl;
  command();

}

//----------------------------------------------------------------------------
//	main() 
//	  - the MAIN
//----------------------------------------------------------------------------
int main(int argc, char *argv[]) {

  signal(SIGINT, handler);

  steps = 20000;
  t 	= 0;

  cout << "Debug mode = " << DEBUG << endl;

  // 1. check args
  if (argc < 2) {
	cout << "usage: " << argv[0] 
	     << " <configfilename> [<network imagefile>] [<start t>]\n";
 	exit(0);
  } 

  // 1.1 save config filename
  strncpy(conffilename,argv[1],strlen(argv[1])+1);

  // 2.1 setup network and allocate memory

  setup(argc,argv,argv[1]);
#ifndef LissomCUI
  spikeact = new REAL* [N];
  for (int i=0; i<N; ++i) spikeact[i] = new REAL [N];
  
  stats       = new simstat(spikeact, N, N, 10);

#endif

  // 2.2 if network imagefilename given, load the network image
  if (argc >= 4) {
	
	cout << "lissom.cc : main() : setting t = " << argv[3] << "\n";
	t = atoi(argv[3]);

	// now go through all hooks so that the simulation environment
	// is the same as when the image was saved
        cout << "lissom.cc : main() : redoing all hooks " << endl;
	environment->rerun_hooks(t-1);	
	cout << "lissom.cc : main() : loading image " << argv[2] << "\n";
	map->net_bb->load(argv[2]);

  }

  // 3. setup input generator
  input_generator = new geninput(input->output,M,M,INP_ORI_BAR,INP_NORM_SUM);
  input_generator->clear_input();


  // initialize inputlist object -- this is used in setup().
  input_spec_list = new inputlist(20,input_generator);

# ifndef LissomCUI

  //
  // 4.1. GUI Interface
  //

  // setup the windows
  mw = new mymain_window(argv[0],MAX(N,M)*GRAN*2,MAX(N,M)*GRAN*2+40);
  
  matw = new matrix_window(*mw,N*GRAN,N*GRAN,0,0,0,60,COLOR,N,N,map->lfp);  

  spikew = new matrix_window(*mw,(int)(MAX(N,M)*GRAN/2.5),
		(int)(MAX(N,M)*GRAN/2.5)
		,MAX(N,M)*GRAN+5,MAX(N,M)*GRAN+5,0,60,
		COLOR,N,N,spikeact);  

  avgw   = new matrix_window(*mw,(int)(MAX(N,M)*GRAN/2.5),
		(int)(MAX(N,M)*GRAN/2.5), 
		MAX(N,M)*GRAN+MAX(N,M)*GRAN/2+5,MAX(N,M)*GRAN+5,0,60,
		COLOR,N,N,map->avg_spiking);  
  avgw->set_scale(2.5);

  inpw = new matrix_window(*mw,MAX(N,M)*GRAN,MAX(N,M)*GRAN,0,
		MAX(N,M)*GRAN+10,0,60,
		COLOR,M,M,input->output);  

  nw = new net_window(*mw,MAX(N,M)*GRAN,MAX(N,M)*GRAN,
		MAX(N,M)*GRAN,0,0,GRAN,
		COLOR,N,N,0.0,1.0,0.0,1.0,map->cogs[0],ideal);

  // weight display
  switch(wgt_disp_flag) {
  case 0 : break;
  case 5:
  case 1 : affdispw = new affw_window("Afferent weight", map,
                        N*(M+1)*WGT_GRAN,N*(M+1)*WGT_GRAN);
         break;
  case 6:
  case 2 : latinhdispw = new latw_window("Lateral inh weight",map,
                        N*(N+1)*WGT_GRAN,N*(N+1)*WGT_GRAN,
                      LATW_INH,10.0,0); 
         break;
  case 3 : latexcdispw = new latw_window("Lateral exc weight",map,
                        N*(N+1)*WGT_GRAN,N*(N+1)*WGT_GRAN,
                        LATW_EXC,10.0);
         break;
  case 4 : latcombdispw = new latw_window("Combined lateral weight",map,
                        N*(N+1)*WGT_GRAN,N*(N+1)*WGT_GRAN,
                        LATW_COMB,10.0);
	 break;
  }

  // Buttons
  runit = new callback_button(*mw,"Run",run,
                mw->width/8,20,mw->width*4/8,mw->height-20);

  saveit = new callback_button(*mw,"Save",save,
		mw->width/8,20,mw->width*5/8,mw->height-20);
 
  testit = new callback_button(*mw,"Test",test,
                mw->width/8,20,mw->width*6/8,mw->height-20);

  affdispit = new callback_button(*mw,"Wgt Disp",affdisp,
                mw->width/8,20,mw->width*3/8,mw->height-20);

  commandit = new callback_button(*mw,"Cmd",command,
                mw->width/8,20,mw->width*2/8,mw->height-20);

  new quit_button(*mw,mw->width/8,20,mw->width*7/8,mw->height-20);

  matw->redraw();
  nw->redraw();

  // start main loop
  mw->main_loop();

  // deallocate stuff
  for (int i=0; i<N; ++i) delete [] spikeact[i];
  delete [] spikeact;

# else

  //
  // 4.2. Text interface
  //
  run();
  save();

# endif

  delete netlog;

}

