#!/usr/bin/perl
#!/usr/bin/perl

$usage = <<'EOF';
usage: genmuafig.pl [-s] <input-spec> <conn-spec> <muaplotfile> <bottom-scale> <#-nrn>

	* Note that the max ppm file or image file size (width) is 1000,
	  and it is assumed that smaller images have an integer
	  multiple which equals 1000.

	-s 	: optional
		  -- simple mode

	input-spec: 
		'i' for input, 'b' for blank, connected by ':'

		i:i:i:i:i  : for 5 consecutive inputs
		i:b:i:b:i:b : for 3 inputs, alternating

	conn-spec:

		'1' for group 1, '2' for group 2, connected by ':'

	muaplotfile:
		
		a png, pdf, or jpg file.

	bottom-scale:

		begin:end:step

		1:500:100	: from 1 to 500, with ticks at every 100

	num-nrn:

		number of neurons

Output: a single pdf file with everything included (uses Tcl).

EOF

if ($ARGV[0] eq "-s") {
	$simple_mode = 1;
	shift;
} else {
	$simple_mode = 0;
}


if ($#ARGV != 4) {
	print $usage;
	print "args = $#ARGV\n";
	exit;
}

$count = 0;
$input_spec = $ARGV[$count++];
$conn_spec = $ARGV[$count++];
$muafile = $ARGV[$count++];
$scale = $ARGV[$count++];
$num_neurons = $ARGV[$count];

#----------------------------------------
# Call genmuaplot.tcl
#----------------------------------------
# set filename    [lindex $argv 0]
# set width       [lindex $argv 1]
# set height      [lindex $argv 2]
# set input_spec  [lindex $argv 3]
# set input_tick_spec  [lindex $argv 4]
# set conn_spec  [lindex $argv 5]
# set conn_tick_spec  [lindex $argv 6]
# set scale_spec  [lindex $argv 7]

$fname = '/tmp/genmuaplot.temp.ppm';
`convert $muafile $fname`;

$sizestr = `identify $fname`;
chop $sizestr;
$sizestr =~ /[\s]+([0-9]+x[0-9]+)[\s]+/;
($width,$height) = split(/x/,$1);

@inps = split(/:/,$input_spec);
$num_inp = $#inps+1;
$input_tick_spec =  "";
$gran=int($num_neurons/($num_inp*1.0));
for ($i=0; $i<=$num_inp; ++$i) {
	if ($i==0) {
		$input_tick_spec .= "1:";
	} elsif ($i==$num_inp) {
		$input_tick_spec .= $num_neurons;
	} else {
		$input_tick_spec .= ($gran*$i).":";
	}
}

($conn_spec_only,$junk) = split(/[ei]/,$conn_spec,2);
@conns = split(/:/,$conn_spec_only);
$num_conn = $#conns+1;
$conn_tick_spec =  "";
$gran=($num_conn==0)?0:int($num_neurons/($num_conn*1.0));
for ($i=0; $i<$num_conn; ++$i) {
	if ($i==0) {
		$conn_tick_spec .= "1:";
	} elsif ($i==$num_conn) {
		$conn_tick_spec .= $num_neurons;
	} else {
		$conn_tick_spec .= ($gran*$i).":";
	}
}

$conn_tick_spec .= "$num_neurons";

if ($num_conn==0) {
	$conn_tick_spec = "1:$num_neurons";
}
	

$scale_spec = $scale;

$execstr = "genmuafig.tcl $fname $width $height $input_spec $input_tick_spec $conn_spec $conn_tick_spec $scale_spec";

print "\n";
print "\n";
print "\n";
print $execstr."\n";

`$execstr`;

$basename = `basename $muafile .ppm`;
chop $basename;

`mv genfig.ps $basename.ps`;
`epstopdf $basename.ps`;
`acroread $basename.pdf`;

