#!/usr/bin/wish

# Configurable area
# set filename "gray.ppm"
# set width 1024
# set height 193
# set input_spec "b:i:b:i:b:i:b"
# set input_tick_spec "0:100:200:300:400:500:600:700"
# set conn_spec "g:b:g:b:g:b"
# set conn_tick_spec "0:100:200:300:400:500:600"
# set scale_spec "1:100:500"

set filename 	[lindex $argv 0]
set iwidth    	[lindex $argv 1]
set iheight	[lindex $argv 2]
set input_spec  [lindex $argv 3]
set input_tick_spec  [lindex $argv 4]
set conn_spec  [lindex $argv 5]
set conn_tick_spec  [lindex $argv 6]
set scale_spec  [lindex $argv 7]
set label_flag  [lindex $argv 8]

# derived values 
set width 	1000
set zoomx	[expr $width/($iwidth)]
set zoomy	$zoomx
set height	[expr int($width/$iwidth*$iheight)]

puts stderr "zoom : x=$zoomx, y=$zoomy"

# set fontsize [expr int(16 * $width/1000)]
# if { $fontsize < 12 } { set fontsize 12 }
set fontsize 18
set smfontsize 16
set tinyfontsize 14
font create myfont -family Helvetica -size $fontsize
font create mysmfont -family Helvetica -size $smfontsize
font create mytinyfont -family Helvetica -size $tinyfontsize
set font "myfont"
set smfont "mysmfont"
set tinyfont "mytinyfont"
set bararearatio 0.06
set barareawidth [expr $bararearatio * $width]
set scaleheight  [expr $bararearatio * $width / 3]
set plotwidth [expr $width + 3.2*$barareawidth]
set plotheight [expr $height + 3.5*$scaleheight]
set barwidth [expr $barareawidth/3]

#----------------------------------------
# First load the image to be annotated
#----------------------------------------
image create photo MUAsmall -file $filename -width $iwidth -height $iheight
image create photo MUA -file $filename -width $width -height $height
MUA copy MUAsmall -from 0 0 $iwidth $iheight -zoom $zoomx $zoomy
puts "Width = $width, Height = $height"

#----------------------------------------
# Create canvas
#----------------------------------------
canvas .plot -width $plotwidth -height $plotheight -background white

set xoffset [expr $barareawidth*1.6]
set yoffset [expr $scaleheight*1.5]

pack .plot -side top -padx 10 -pady 10

#----------------------------------------
# Create buttons
#----------------------------------------
button .psdump -command { \
        .plot postscript -file genfig.ps ; \
        } -text "PS Dump"
pack .psdump -side top;

button .quit -command { exit } -text "Quit"
pack .quit -side top -padx 10 -pady 10

#----------------------------------------
# Add image to canvas
#----------------------------------------
.plot create image $xoffset $yoffset -image MUA -anchor nw
# draw border
.plot create rectangle $xoffset $yoffset \
		[expr $xoffset+$width] [expr $height+$yoffset] -outline black


#----------------------------------------
# Add input spec to the canvas
#----------------------------------------

# Parse input spec string
set inputs [split $input_spec ":"]
set ticks   [split $input_tick_spec ":"]
set numinp [llength $inputs]

# .plot create rectangle [expr $barwidth] [expr $yoffset]\
#	[expr $barwidth*2] \
#	[expr $yoffset + $height] -outline black

.plot create text [expr $barwidth]  [expr $yoffset*1.5 + $height] \
	-text "Input" -font $smfont  -anchor w

for {set i 0} {$i<=$numinp} {incr i} {

	if { $i == $numinp } {
		set ycorner [expr $height - ($i + 1) * $height/$numinp + $yoffset]
		set ybegin [expr $yoffset]
	} else {
		set ycorner [expr $height - ($i + 1) * $height/$numinp + $yoffset]
		set ybegin [expr $height - $i * $height/$numinp + $yoffset]
	}

	
	if { $i == $numinp } {	# only plot text
		.plot create text [expr $barwidth*3] $ybegin \
			-text [lindex $ticks [expr $i]] -anchor w -font $font
		break;
	}

	if { [lindex $inputs $i] == "i" } {
		# draw vertical bar	
		.plot create rectangle [expr $barwidth*1.6]  $ybegin \
			[expr $barwidth*2]  $ycorner \
			-fill black -outline black
	} else {
		# draw vertical bar	
		.plot create rectangle [expr $barwidth*1.6]  $ybegin \
			[expr $barwidth*2]  $ycorner \
			-fill white -outline black 
	}

	.plot create text [expr $barwidth*3] $ybegin\
		-text [lindex $ticks [expr $i]] -anchor w -font $font 
}

#----------------------------------------
# Add connection spec to the canvas
#----------------------------------------

# Parse connection spec string
set inputs [split $conn_spec ":"]
set ticks   [split $conn_tick_spec ":"]
set speclen [llength $inputs]

set numinp 0
for {set i 0} {$i<$speclen} {incr i} {
	if {([lindex $inputs $i] == "g") || ([lindex $inputs $i] == "G")} {
		incr numinp
	}
}
puts stderr "numinp = $numinp"


# common values
set xexcbegin [expr $plotwidth-2*$barwidth]
set xexccorner [expr $plotwidth-1.5*$barwidth-2]

set xinhbegin [expr $plotwidth-1.5*$barwidth+2]
set xinhcorner [expr $plotwidth-$barwidth]

# plot place holder

set exc_tagged 0
set inh_tagged 0

set numticks [llength $ticks]

for {set i 0} {$i<=$numticks} {incr i} {
	if {$numticks>1} {
		set ypos [expr $height - $i * $height/($numticks-1)+ $yoffset]
		puts stderr "tick num $i"
		puts stderr [lindex $ticks $i]
		puts stderr "tick pos $ypos"
	} else {
		puts stderr "tick num $i"
		puts stderr [lindex $ticks $i]
		set ypos [expr $height - ($height * $i) + $yoffset ]
		puts stderr "tick pos $ypos"
	}
	.plot create text [expr $xoffset + $barwidth/2 + $width ] $ypos \
		-text [lindex $ticks $i] -anchor w -font $font

}

set two_exc_flag 0

# parse and plot connection spec 
for {set i 0} {$i<=$speclen} {incr i} {

	set spec [lindex $inputs $i]

	if { $numinp != 0 } {
	  if { $i == $numinp } {
		set ycorner [expr $height - ($i + 1) * $height/$numinp + $yoffset ]
		set ybegin [expr $yoffset]
	  } else {
		set ycorner [expr $height - ($i + 1) * $height/$numinp + $yoffset ]
		set ybegin [expr $height - $i * $height/$numinp + $yoffset]
	  }
	}

	switch -glob $spec {

	  g	{; # exc group
		set xexcbegin2 [expr $xexcbegin-$barwidth/2]
		set xexccorner2 [expr $xexccorner-$barwidth/2]
		.plot create rectangle $xexcbegin2 $ybegin $xexccorner2 $ycorner \
			-fill black -outline black
		puts stderr "* g"
		set exc_tagged 1
		set two_exc_flag 1
	  }

	  G 	{; # exc group2
		.plot create rectangle $xexcbegin $ybegin $xexccorner $ycorner \
			-fill black -outline black
		puts stderr "* G"
		set exc_tagged 1
	  }

	  e*	{; # e,<radius> exc radius
		puts stderr "* e"
		puts stderr "exc($spec)"
		set radarray [split $spec ","]
		set rad [lindex $radarray 1]
		set radmax [lindex $radarray 2]
		puts stderr "rad: $rad, radmax: $radmax"
		set radlen [expr $height*$rad*2/$radmax]
		set radbegin [expr $yoffset + ($height - $radlen)/2.0]
		set radend   [expr $radbegin + $radlen]
		.plot create rectangle $xexcbegin $radbegin \
			$xexccorner $radend -fill black -outline black
		set exc_tagged 1
	  }

	  i*	{; # inh radius
		puts stderr "* i"
		puts stderr "inh($spec)"
		set radarray [split $spec ","]
		set rad [lindex $radarray 1]
		set radmax [lindex $radarray 2]
		puts stderr "rad: $rad, radmax: $radmax"
		set radlen [expr $height*$rad*2/$radmax]
		set radbegin [expr $yoffset + ($height - $radlen)/2.0]
		set radend   [expr $radbegin + $radlen]
		.plot create rectangle $xinhbegin $radbegin \
			$xinhcorner $radend -fill black -outline black
		set inh_tagged 1
	  }	

	  n {

.plot create rectangle $xinhbegin $yoffset \
	$xinhcorner [expr $yoffset + $height] \
	-outline black 
.plot create rectangle $xexcbegin $yoffset \
	$xexccorner [expr $yoffset + $height] \
	-outline black 


	  }

	}


}

if { $two_exc_flag == 0 } {
	set tag "  Exc|Inh"
	set tfont $smfont
} else {
	set tag "  E1|E2|Inh"
	set tfont $tinyfont
}

.plot create text [expr $plotwidth-3.5*$barwidth] [expr $yoffset*0.5] \
	-text $tag -font $tfont  -anchor w

.plot create text [expr $plotwidth-4.0*$barwidth] [expr $yoffset*1.5 + $height] \
	-text "  Sample" -font $tinyfont  -anchor w
.plot create text [expr $plotwidth-4.0*$barwidth] [expr $yoffset*1.9+ $height] \
	-text "  Lat.Conn." -font $tinyfont  -anchor w

if { $inh_tagged == 0 } {
set blank_color "\#dddddd"
.plot create rectangle $xinhbegin $yoffset \
	$xinhcorner [expr $yoffset + $height] \
	-outline black 

}

if { $exc_tagged == 0 } {
set blank_color "\#dddddd"
.plot create rectangle $xexcbegin $yoffset \
	$xexccorner [expr $yoffset + $height] \
	-outline black 
}


# .plot create text [expr $width+$barwidth*5.5] [expr $yoffset*3 + $height] -text $connstr -font $smfont  -anchor w

#----------------------------------------
# Add scale to bottom of the canvas
#----------------------------------------

set onescaleheight [expr $scaleheight/4]
set scale_param [split $scale_spec ":"]
set begin [lindex $scale_param 0]
set steps [lindex $scale_param 1]
set end [lindex $scale_param 2]
set endidx [expr $end/$steps]
set gran [expr $width/(($end-$begin+1)/$steps)]

puts "Gran = $gran"

.plot create line \
	$xoffset [expr $height+$onescaleheight*2 + $yoffset ] \
	[expr $xoffset + $width] [expr $height+$onescaleheight*2 + $yoffset]  \
	-fill black -width 2

set ticktop [expr $height+$onescaleheight*2 + $yoffset ]
set tickbottom [expr $height+$onescaleheight*4 + $yoffset ]
set scalepos [expr $height+$onescaleheight*6 + $yoffset ]

for {set i 0} {$i<=$endidx} {incr i} {

	if {$i == $endidx} {
		set xcorner [expr $xoffset + $width]
	} else {
		set xcorner  [expr $xoffset + $gran*$i]
	}
	
	.plot create line \
		$xcorner $ticktop \
		$xcorner $tickbottom \
		-fill black -width 2
	.plot create text \
		$xcorner $scalepos \
		-text [expr $i * $steps] -font $font
} 

update;

.plot postscript -file genfig.ps;

exit;
