#!/usr/bin/perl

# process (phi,theta,d) statistics dump from the command
# dump_wgt_or_stat in slissom.
#	Blank out every other track. Just simply multiply rho by 2.

$distance_binsize = 5;

if ($#ARGV != 3) {

	print "usage: prowgtstat.pl -theta|-thetapolar|-prob|-max|-sum|-maxpolar|-probpolar|-sumpolar <size> <thresh> <or_stat_dump_filename>\n";
	print "		-max is the default \n";
	print <<EOF;
	-theta : print the most probable theta -- 2d matrix
	-thetapolar : print the most probable theta -- polar coordinate
	-prob  : print the probability -- 2d matrix
	-max   : prob of most probable theta -- 2d matrix
	-sum	: sum?
	-maxpolar : prob of most probable theta -- polar coordinate
	-probpolar : prob of most probable theta -- polar coordinate
	-sumpolar : ??
	<size> : only used for 2d matrix -- resulting size

	* multiplies 10 to theta -- this is for subru's data
EOF

	exit;

}

$theta_output_flag = ($ARGV[0] eq "-theta")?1:0;
$theta_polar_output_flag = ($ARGV[0] eq "-thetapolar")?1:0;
$max_output_flag = ($ARGV[0] eq "-max")?1:0;
$max_polar_output_flag = ($ARGV[0] eq "-maxpolar")?1:0;
$sum_output_flag = ($ARGV[0] eq "-sum")?1:0;
$prob_polar_output_flag = ($ARGV[0] eq "-probpolar")?1:0;
$sum_polar_output_flag = ($ARGV[0] eq "-sumpolar")?1:0;

# init matrix
$N = $ARGV[1];
for ($i=0; $i<$N; ++$i) {
	for ($j=0; $j<$N; ++$j) {
		$matrix[$i][$j] = 0;
	}
}

$thresh = $ARGV[2];

# load file and proces
open(FP,"$ARGV[3]");

$sum = 0.0;
$alternate_zero_degree = 0;
$alternate_count = 0;
$max_d = 0;
while(<FP>) {
	chop;
	# Get stat
	($phi,$d,$theta) = split(/\ /,$_);

	$theta *= 10.0;

	$d = int($d/$distance_binsize);

	if ($d>$max_d) { $max_d = $d; }

	# Fix side effect of only having 0 to 170 degree
	if (0) {
		if ($alternate_count%2==0) {
			$theta = 180;
		} 
		$alternate_count ++;
	}


	# update phi,d,theta count
	$phi_d{"$phi $d $theta"} ++;
	$phi_d_sum{"$phi $d"} ++;
	$sum ++;
	$sum_prob{"$phi $d"} ++;
	
	# update max prob theta (theta that has the maximum count)
	if (exists($max_prob_theta{"$phi $d"})) {
		$max_theta = $max_prob_theta{"$phi $d"};
	} else {
		$max_theta = 0;
	}
	if ($phi_d{"$phi $d $theta"}>$phi_d{"$phi $d $max_theta"}) {
		$max_prob_theta{"$phi $d"} = $theta;
		$max_prob{"$phi $d"} = $phi_d{"$phi $d $theta"};
	}
	# $sum_prob{"$phi $d"} += $phi_d{"$phi $d $theta"};


	# Dirty hack : rotate 180 and add that too.

	$phi = ($phi+180)%360;

	# update phi,d,theta count
	$phi_d{"$phi $d $theta"} ++;
	$phi_d_sum{"$phi $d"} ++;
	$sum ++;
	$sum_prob{"$phi $d"} ++;
	
	# update max prob theta (theta that has the maximum count)
	if (exists($max_prob_theta{"$phi $d"})) {
		$max_theta = $max_prob_theta{"$phi $d"};
	} else {
		$max_theta = 0;
	}
	if ($phi_d{"$phi $d $theta"}>$phi_d{"$phi $d $max_theta"}) {
		$max_prob_theta{"$phi $d"} = $theta;
		$max_prob{"$phi $d"} = $phi_d{"$phi $d $theta"};
	}
	# $sum_prob{"$phi $d"} += $phi_d{"$phi $d $theta"};


	$phi = ($phi+180)%360;
}
close(FP);

print STDERR "ALT_COUNT (0 vs 180) = $alternate_count\n";

$phidsum = 0.0;
foreach $key (keys(%sum_prob)) {
	$phidsum += $sum_prob{$key};
}
print STDERR " SUM = $sum \n";
print STDERR " Sigma_{phi} Sigma_{d} Sigma_{theta} = $phidsum \n";
  

if ($max_output_flag) {
	foreach $key (keys(%max_prob_theta)) {
		print "$key $max_prob_theta{$key}\n";
	}
	exit;
}


# print result in a NxN matrix
$half_N = int($N/2);

$prob_sum = 0.0;
$min_prob = 999;
foreach $key (keys(%max_prob_theta)) {
	($phi,$d) = split(/\ /,$key);

  	if ($max_polar_output_flag) {
			print "$phi $d ".$max_prob{"$phi $d"}."\n";
	} elsif ($prob_polar_output_flag) {
			$prob = $sum_prob{"$phi $d"}/($sum*1.0);
			if ($prob<$min_prob) { $min_prob = $prob; }
			$prob_sum += $prob;
			print "$phi $d $prob\n";
	} elsif ($sum_polar_output_flag) {
			$prob = $sum_prob{"$phi $d"};
			$prob_sum += $prob;
			print "$phi $d $prob\n";
	} elsif ($theta_polar_output_flag) {
			$theta = $max_prob_theta{$key};
			$or_val = 1.0-$theta/180.0;
			print "$phi $d $or_val\n";
	} else {
		# For cartesian matrix output
		$t = 3.141592*$phi/180.0;
		$x = int($d*cos($t)+0.5)+$half_N;
		$y = int($d*sin($t)+0.5)+$half_N;
		$theta = $max_prob_theta{$key};
		if ($max_prob{"$phi $d"}>$thresh) {
			if ($theta_output_flag || $theta_polar_output_flag) {
				$matrix[$x][$y] = 1.0-$theta/180.0;
				
			} elsif ($sum_output_flag) {
				$matrix[$x][$y] = $sum_prob{"$phi $d"};
			} else {
				$matrix[$x][$y] = $max_prob{"$phi $d"};
			}
		}
	}
} 

# fill in ------ disabled
if (0) {
if ($prob_polar_output_flag) {
	for ($i=0; $i<360; $i++) {
		for ($d=0; $d<=$max_d; ++$d) {
			if(!exists($max_prob_theta{"$phi $d"})) {
				print "$i $d $min_prob\n";
			}
		}
	}
}	
}
		
		

print STDERR "RECOUNTED_SUM = $prob_sum \n";

if ($max_polar_output_flag || $prob_polar_output_flag) {
	exit;
}

for ($j=0; $j<$N; ++$j) {
	for ($i=0; $i<$N; ++$i) {
		print $matrix[$i][$j]." ";
	}
	print "\n";
}

