## Copyright (C) 2000 John W. Eaton
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## usage: yimage(X,M,zoom)
##
## Returns scaled matrix X to the colormap M and display it with the given
## colormap. 
##
##	M can be one of these (include the double quotes). 
##	  "graymap" "graymap2" "graymap3" 
##	  "hotmap" "hotmap2" "hotmap3"
##	  "coolmap" "coolmap2" "coolmap3"
##	  "hsvmap" "hsvmap2" "wackymap" "wackymap2"
##	
##	  - the ones with suffix "2" are the inverse colormap.
##	  - graymap3 is a bit different: black->white->black.
##	  - to view the full range, do something like this (change the
##	    colormap each time you run yimage):
##		
##		cbar = [1:64];
##		yimage(cbar,"hsvmap",15);
##
##	zoom is an integer zoom factor.
##	
##	Both M and zoom are optional. Default values are "graymap" and 4,
##	respectively.
##
##	For example, 
##
##		yimage(X,"graymap",1)
##
## See also: image, imagesc, colormap

## Author: Yoonsuck Choe <yschoe@cs.utexas.edu>

function scaled = yimage (X,mode,zoom)

  if (nargin ==0)
    usage ("yimage (X,mode,zoom)");
  elseif (nargin==1) 
    mode="graymap";
    zoom=4;
  elseif (nargin==2) 
    zoom=4;
  elseif (nargin>3)
    usage ("yimage (X,mode,zoom)");
  endif


  # select colormap : hmm.. how do I use strcat to make this
  # less ugly??
  if (strcmp(mode,"hsvmap"))
  	load "INSTDIR/hsvmap.dat"
  elseif (strcmp(mode,"hsvmap2"))
  	load "INSTDIR/hsvmap.dat"
	ycmap(1,:) = [1,1,1];	
  elseif (strcmp(mode,"hsvmap5"))
  	load "INSTDIR/hsvmap.dat"
	ycmap(1,:) = [0,0,0];	
  elseif (strcmp(mode,"hsvmap3"))
  	load "INSTDIR/hsvmap.dat"
	ycmap = ycmap(1:215,:);
 	ycmap = flipud(ycmap);
	ycmap = [ 1 1 1; ycmap ];
  elseif (strcmp(mode,"hsvmap4"))
  	load "INSTDIR/hsvmap.dat"
	ycmap = ycmap(1:200,:);
 	ycmap = flipud(ycmap);
	ycmap = [ 1 1 1; ycmap ];
  elseif (strcmp(mode,"hotmap"))
  	load "INSTDIR/hotmap.dat"
  elseif (strcmp(mode,"hotmap2"))
  	load "INSTDIR/hotmap2.dat"
  elseif (strcmp(mode,"hotmap3"))
  	load "INSTDIR/hotmap3.dat"
  elseif (strcmp(mode,"coolmap"))
  	load "INSTDIR/coolmap.dat"
  elseif (strcmp(mode,"coolmap2"))
  	load "INSTDIR/coolmap2.dat"
  elseif (strcmp(mode,"coolmap3"))
  	load "INSTDIR/coolmap3.dat"
  elseif (strcmp(mode,"graymap2"))
  	load "INSTDIR/graymap2.dat"
  elseif (strcmp(mode,"graymap3"))
  	load "INSTDIR/graymap3.dat"
  elseif (strcmp(mode,"wackymap"))
  	load "INSTDIR/wackymap.dat"
  elseif (strcmp(mode,"wackymap2"))
  	load "INSTDIR/wackymap2.dat"
  else 
  	load "INSTDIR/graymap.dat"
  endif

  # scale matrix
  # cmap_len = length(ycmap);
  # Xmax = max(max(X)); 
  # Xmin = min(min(X)); 
  # scaled = (X.-Xmin)./(Xmax-Xmin).*cmap_len;
  
  # plot image
  colormap(ycmap);
  imagesc(X,zoom); 

endfunction
